class MapController < ApplicationController
  
  def filter

	#convert check_box value to true/false	
	@mental_health = '1' == params[:mental_health]
	@substance_abuse = '1' == params[:substance_abuse]

	#set our session variable to hold the category checkbox values. This way we have access to them across the entire application.
	session[:filters] = []
	session[:filters] << ('1' == params[:mental_health])
	session[:filters] << ('1' == params[:substance_abuse])

 	#We have lost the reference to the map from the Ruby code but in JavaScript, 
  	#the map is still globally accessible with the map variable.
  	#We use the line below to indicate that all the methods called on the @map 
  	#variable in the RJS code must be called on the map variable in the JavaScript code.
  	@myGroup = Variable.new("myGroup")
	@map = Variable.new("map")

	locations = []
	#read all the locations stored in the database and filter if needed
	if session[:loc].blank?
		if @mental_health and @substance_abuse
			locations = Location.find(:all, :conditions => "category_1 = 'Mental Health' OR category_1 = 'Substance Abuse'")
		elsif @mental_health
			locations = Location.find(:all, :conditions => "category_1 = 'Mental Health'")
		elsif @substance_abuse
			locations = Location.find(:all, :conditions => "category_1 = 'Substance Abuse'")
		end
	else
		if @mental_health and @substance_abuse
			locations = Location.find(:all, :origin => [session[:loc].lat, session[:loc].lng], :within => session[:dist].to_i)
		elsif @mental_health
			locations = Location.find(:all, :origin => [session[:loc].lat, session[:loc].lng], :within => session[:dist].to_i, :conditions => "category_1 = 'Mental Health'")
		elsif @substance_abuse
			locations = Location.find(:all, :origin => [session[:loc].lat, session[:loc].lng], :within => session[:dist].to_i, :conditions => "category_1 = 'Substance Abuse'")
		end
	end

  	num_locations = locations.size

	#message to be displayed on results page. Green message indicates results indicates locations were found and red indicates no locations.
    @message = "#{num_locations} #{num_locations == 1 ? " location" : " locations"} found within #{session[:dist]} #{session[:dist].to_i == 1 ? " mile" : " miles"} of #{session[:zip]}."
  	@message_color = "#{num_locations > 0 ? "green" : "red"}"

	#call the get_locations with the locations found based on the search criteria. Returns an array of the google maps markers to be placed on the map.
	results = Map.get_locations(locations)

	@side_bar_html = results[1]	#information for left side bar
	@markers = results[0]	#marker information
	
  end
  
  def index
  
  end
  
  def find_ajax

  	#lookup the geolocation of this zip code
	sleep(1) #ensures that lookups occur no faster than once per second (limit set by Google API)
  	loc=GeoKit::Geocoders::MultiGeocoder.geocode(params[:zip])
	session[:loc] = loc
	session[:dist] = params[:dist]
  	session[:zip] = params[:zip]
  	
  	#We have lost the reference to the map from the Ruby code but in JavaScript, 
  	#the map is still globally accessible with the map variable.
  	#We use the line below to indicate that all the methods called on the @map 
  	#variable in the RJS code must be called on the map variable in the JavaScript code.
  	@map = Variable.new("map")
  	@myGroup = Variable.new("myGroup")

	locations = []

	#read all the locations stored in the database and filter if needed
	if session[:filters][0] and session[:filters][1]
		locations = Location.find(:all, :origin => [session[:loc].lat, session[:loc].lng], :within => session[:dist].to_i)
	elsif session[:filters][0]
		locations = Location.find(:all, :origin => [session[:loc].lat, session[:loc].lng], :within => session[:dist].to_i, :conditions => "category_1 = 'Mental Health'")
	elsif session[:filters][1]
		locations = Location.find(:all, :origin => [session[:loc].lat, session[:loc].lng], :within => session[:dist].to_i, :conditions => "category_1 = 'Substance Abuse'")
	end
	
  	num_locations = locations.size
    @message = "#{num_locations} #{num_locations == 1 ? " location" : " locations"} found within #{session[:dist]} #{session[:dist].to_i == 1 ? " mile" : " miles"} of #{session[:zip]}."
  	@message_color = "#{num_locations > 0 ? "green" : "red"}"

	#call the get_locations with the locations found based on the search criteria. Returns an array of the google maps markers to be placed on the map.
	results = Map.get_locations(locations)

	#if no locations were found, display the words "Non found" in the side bar. Else display the names of the locations found.
	if locations.empty?
		@side_bar_html = "<br/>&nbsp;None found."
	else
		@side_bar_html = results[1]
	end
	
	
	@markers = results[0]	#marker information

  	#map zoom level (0-19); larger means more zoomed in, smaller means more zoomed out
  	@zoom = 13
  	
  	#zoom the map based on the distance between the locations
	unless locations.empty?
		@sorted_latitudes = locations.collect(&:latitude).compact.sort
		@sorted_longitudes = locations.collect(&:longitude).compact.sort
	
		#find the rough middle of the marker group
		tmp1 = (@sorted_latitudes.first + @sorted_latitudes.last) / 2.0 
		tmp2 = (@sorted_longitudes.first + @sorted_longitudes.last) / 2.0

		dist1 = @sorted_latitudes.first.abs - @sorted_latitudes.last.abs
		dist2 = @sorted_longitudes.first.abs - @sorted_longitudes.last.abs

		if (dist1) > 0.2 || (dist2) > 0.2
			@zoom = 11
		elsif (dist1) > 0.01 || (dist2) > 0.01
			@zoom = 14
		end

	end

	#center the map based on the results of the zip code geocoding
  	if loc.success == "false"
  		@center = GLatLng.new([ 39, -98 ])	#if an unknown zip, center of U.S.
	elsif locations.empty?
		@center = GLatLng.new([ loc.lat, loc.lng ])	#if a known zip but no locations are near it, center on zip found
	else
		@center = GLatLng.new([ tmp1, tmp2 ])	#if a known zip and locations exists near it, center on middle of marker group
  	end
	
  end
  
  def find
  	
	#set our session variable to hold the category checkbox values and search criteria. This way we have access to them across the entire application.
	session[:zip] = params[:zip]
	session[:dist] = params[:dist]
	session[:filters] = []
	
	session[:filters] << ('1' == params[:mental_health])
	session[:filters] << ('1' == params[:substance_abuse])
	
	#redirect to the results page (which runs the results method below)
	render :update do |page|
		page.redirect_to :controller => "map", :action => "results"
	end

  end
  
  def results
  
  	@message = ""

	#if a user happend to go directly to the results page without an initial search, redirect to main index page
  	if session[:zip].blank?
  		redirect_to :action => 'index'
  	end
    sleep(1) #ensures that lookups occur no faster than once per second (limit set by Google API)
    loc=GeoKit::Geocoders::MultiGeocoder.geocode(session[:zip])

	session[:loc] = loc
  
  	locations = []
  
  	@filters = []
  	@filters << session[:filters][0]
  	@filters << session[:filters][1]
  	
	#read all the locations stored in the database and filter if needed
	if session[:filters][0] and session[:filters][1]
		locations = Location.find(:all, :origin => [session[:loc].lat, session[:loc].lng], :within => session[:dist].to_i)
	elsif session[:filters][0]
		locations = Location.find(:all, :origin => [session[:loc].lat, session[:loc].lng], :within => session[:dist].to_i, :conditions => "category_1 = 'Mental Health'")
	elsif session[:filters][1]
		locations = Location.find(:all, :origin => [session[:loc].lat, session[:loc].lng], :within => session[:dist].to_i, :conditions => "category_1 = 'Substance Abuse'")
	end
  
  	num_locations = locations.size

	#message to be displayed on results page. Green message indicates results indicates locations were found and red indicates no locations.
    @message = "#{num_locations} #{num_locations == 1 ? " location" : " locations"} found within #{session[:dist]} #{session[:dist].to_i == 1 ? " mile" : " miles"} of #{session[:zip]}."
  	@message_color = "#{num_locations > 0 ? "green" : "red"}"

  	#create a map on the page
	@map = GMap.new("map_div")
	
	#set the type of map (small/large)
	@map.control_init(:large_map => true, :map_type => true)

	#allow for scroll wheel access
	@map.interface_init(:scroll_wheel_zoom => true)

	@locMarkers = {}
	@markers = []

	#if a known zip and locations exists near it, center on middle of marker group
	results = Map.get_locations(locations)

	#if no locations were found, display the words "Non found" in the side bar. Else display the names of the locations found.
	if locations.empty?
		@side_bar_html = "<br/>&nbsp;None found."
	else
		@side_bar_html = results[1]
	end
	
	#call the get_locations with the locations found based on the search criteria. Returns an array of the google maps markers to be placed on the map.
	@markers = results[0]	#marker information

	#go through the marker array and create a hash out of it
	i = 0
	while (i < @markers.size)
		@locMarkers.merge!({(i+1) => @markers[i]})
		i += 1
	end

	@group = GMarkerGroup.new(true,@locMarkers)
	@map.overlay_global_init(@group,"myGroup") #add the MarkerGroup to the map

	#center the map based on the results of the zip code geocoding
	if loc.success == "false"
		@map.center_zoom_init([39, -98],13)	#if an unknown zip, center of U.S.
	elsif locations.empty?
		@map.center_zoom_init([loc.lat, loc.lng],13)	#if a known zip but no locations are near it, center on zip found
	else
		@map.record_init @group.center_and_zoom_on_markers	#if a known zip and locations exists near it, center on middle of marker group
	end
	
  end

end

